library ieee;
   use ieee.std_logic_1164.all;
   use IEEE.STD_LOGIC_ARITH.ALL;
   use IEEE.STD_LOGIC_UNSIGNED.ALL;
   use ieee.numeric_std.all;

use work.pkg_vectrex.all;
use work.pkg_d2a.all;

entity vectrex_de1 is
  port(
	-- clocks
	clock_24    : in    std_logic_vector(1 downto 0);	
	clock_27    : in    std_logic; 						
    clock_50    : in    std_logic; 						
    ext_clock   : in    std_logic; 						
    
    -- push button keys
	key         : in    std_logic_vector(3 downto 0);  

	-- LEDS
	ledg        : out std_logic_vector(7 downto 0);
	ledr        : out std_logic_vector(9 downto 0);
		
	--Switches
	sw          : in  std_logic_vector(9 downto 0);
	
	--GPIOs
	gpio_0      : inout   std_logic_vector(35 downto 0);	
	gpio_1      : out   std_logic_vector(35 downto 0);
	
	--7 Segment 
	hex0	    : out std_logic_vector (0 to 6);
	hex1	    : out std_logic_vector (0 to 6);
	hex2	    : out std_logic_vector (0 to 6);
	hex3	    : out std_logic_vector (0 to 6)	
	);
end vectrex_de1;

architecture my_vectrex of vectrex_de1 is
  -----------------------------------------------------------------------------
  -- Signals
  -----------------------------------------------------------------------------

signal sys_clk       	: std_logic; 
signal pwm_clk    	: std_logic;
signal player1_buttons:		STD_LOGIC_VECTOR(3 downto 0);
signal player2_buttons:		STD_LOGIC_VECTOR(3 downto 0);
signal player1_x:		STD_LOGIC_VECTOR(7 downto 0);
signal player1_y:		STD_LOGIC_VECTOR(7 downto 0);
signal player2_x:		STD_LOGIC_VECTOR(7 downto 0);
signal player2_y:		STD_LOGIC_VECTOR(7 downto 0);
signal psxbuttons:	STD_LOGIC_VECTOR(15 downto 0);
signal clk_ibuf:		STD_LOGIC;
signal clk_dlla_op0:	STD_LOGIC;
signal clk_dlla_div:	STD_LOGIC;
signal clk_dlla_2x:	STD_LOGIC;
signal reset_h:		STD_LOGIC;
signal reset_l:		STD_LOGIC;
signal clk_104:		STD_LOGIC;
signal clk_12:			STD_LOGIC;
signal clke_psx:		STD_LOGIC;
signal clkc_psx:		STD_LOGIC_VECTOR(3 downto 0);
signal clk_pwm:		STD_LOGIC;
signal xval:			STD_LOGIC_VECTOR(9 downto 0);
signal yval:			STD_LOGIC_VECTOR(9 downto 0);
signal xval_filt:			STD_LOGIC_VECTOR(9 downto 0);
signal yval_filt:			STD_LOGIC_VECTOR(9 downto 0);
signal xval_pwm:			STD_LOGIC_VECTOR(9 downto 0);
signal yval_pwm:			STD_LOGIC_VECTOR(9 downto 0);
signal xval_inv:			STD_LOGIC_VECTOR(9 downto 0);
signal yval_inv:			STD_LOGIC_VECTOR(9 downto 0);
signal zval	:			STD_LOGIC_VECTOR(7 downto 0);
signal rgb	:			STD_LOGIC_VECTOR(2 downto 0);
signal soundval:			STD_LOGIC_VECTOR(7 downto 0);
signal soundout:			STD_LOGIC;
signal cpu_reset_h:		STD_LOGIC;

component pll IS
	PORT
	(
		inclk0		: IN STD_LOGIC  := '0';
		c0			: OUT STD_LOGIC ;
		c1			: OUT STD_LOGIC
	);
end component;

component vectrex is
  port(
		cpu_areset   		: in    std_logic;
		cpu_ahold    		: in    std_logic;
		cpu_anmi     		: in    std_logic;
		cpu_clk		 		: in    std_logic; --12 MHz
		dbg    		 		: out std_logic_vector(15 downto 0);
		analog_sound_out    : out std_logic_vector(7 downto 0);
		analog_x_out    	: out std_logic_vector(9 downto 0);
		analog_y_out    	: out std_logic_vector(9 downto 0);
		analog_z_out    	: out std_logic_vector(7 downto 0);
		p1_x				: in std_logic_vector(7 downto 0);
		p1_y				: in std_logic_vector(7 downto 0);
		p1_buttons		 	: in std_logic_vector(3 downto 0);
		p2_x				: in std_logic_vector(7 downto 0);
		p2_y				: in std_logic_vector(7 downto 0);
		p2_buttons		 	: in std_logic_vector(3 downto 0)
	);
end component;

begin

reset_h 	<= '0';
cpu_reset_h <= '0';
clk_pwm 	<= pwm_clk;
clk_12		<= sys_clk;
xval_pwm<=xval xor "1000000000";
yval_pwm<=yval xor "1000000000";
--yval_pwm<=xval_inv; --board swaps x and y
--xval_pwm<=yval_inv;
--gpio_1(6)<=zval(7 downto 4);
player1_buttons<="1111";
player2_buttons<="1111";
     
my_pll : pll
	PORT MAP
	(
		inclk0		=> clock_24(0),
		c0			=> sys_clk,
		c1			=> pwm_clk
	);

pwmxh: pwmgen_8 port map (
		inval => xval_pwm(9 downto 6),
		output => gpio_1(1),
		clk => clk_pwm,
		reset => reset_h
	);
	pwmxl: pwmgen_8 port map (
		inval => xval_pwm(5 downto 2),
		output => gpio_1(2),
		clk => clk_pwm,
		reset => reset_h
	);
	pwmyh: pwmgen_8 port map (
		inval => yval_pwm(9 downto 6),
		output => gpio_1(3),
		clk => clk_pwm,
		reset => reset_h
	);
	pwmyl: pwmgen_8 port map (
		inval => yval_pwm(5 downto 2),
		output => gpio_1(4),
		clk => clk_pwm,
		reset => reset_h
	);
	
	audioda: deltasigma generic map (
			width => 8
		) port map (
			inval => soundval,
			output => gpio_1(0),
			clk => clk_pwm,
			reset => cpu_reset_h
	);

	myvectrex: vectrex port map (
		cpu_areset => cpu_reset_h,
		cpu_ahold => '0',
		cpu_anmi => '0',
		cpu_clk => clk_12,
		analog_sound_out => soundval,
		analog_x_out => xval,
		analog_y_out => yval,
		analog_z_out => zval,
		p1_x => player1_x,
		p1_y => player1_y,
		p1_buttons => player1_buttons,
		p2_x => player2_x,
		p2_y => player2_y,
		p2_buttons => player2_buttons,
		dbg => open
	);
	
end my_vectrex;

