library IEEE;
use IEEE.STD_LOGIC_1164.all;

package pkg_vectrex is



component vectrex
  port(
		cpu_areset   : in    std_logic;
		cpu_ahold    : in    std_logic;
		cpu_anmi     : in    std_logic;
		cpu_clk		: in    std_logic; --12 MHz
		dbg    : out std_logic_vector(15 downto 0);
		analog_sound_out    : out std_logic_vector(7 downto 0);
		analog_x_out    : out std_logic_vector(9 downto 0);
		analog_y_out    : out std_logic_vector(9 downto 0);
		analog_z_out    : out std_logic_vector(7 downto 0);
		p1_x				 : in std_logic_vector(7 downto 0);
		p1_y				 : in std_logic_vector(7 downto 0);
		p1_buttons		 : in std_logic_vector(3 downto 0);
		p2_x				 : in std_logic_vector(7 downto 0);
		p2_y				 : in std_logic_vector(7 downto 0);
		p2_buttons		 : in std_logic_vector(3 downto 0);
		p1_buttons_out: out  std_logic_vector(3 downto 0);
		p2_buttons_out: out  std_logic_vector(3 downto 0)
	);
end component;


-----------------------------------------------------------------
--
-- CPU09 CPU core
--
-----------------------------------------------------------------

component cpu09
  port (    
		clk      :	in std_logic;                     -- E clock input (falling edge)
		rst      :  in std_logic;                     -- reset input (active high)
		vma      : out std_logic;                     -- valid memory address (active high)
      lic      : out std_logic;                     -- last instruction cycle (active high)
      ifetch   : out std_logic;                     -- instruction fetch cycle (active high)
      ba       : out std_logic;                     -- bus available (high on sync wait or DMA grant)
      bs       : out std_logic;                     -- bus status (high on interrupt or reset vector fetch or DMA grant)
		addr     : out std_logic_vector(15 downto 0); -- address bus output
		rw       : out std_logic;                     -- read not write output
	   data_out : out std_logic_vector(7 downto 0);  -- data bus output
	   data_in  :  in std_logic_vector(7 downto 0);  -- data bus input
		irq      :  in std_logic;                     -- interrupt request input (active high)
		firq     :  in std_logic;                     -- fast interrupt request input (active high)
		nmi      :  in std_logic;                     -- non maskable interrupt request input (active high)
		halt     :  in std_logic;                     -- halt input (active high) grants DMA
		hold     :  in std_logic                      -- hold input (active high) extend bus cycle
  );
end component;

----------------------------------------
--
-- 8KByte Block RAM as Main ROM
--
----------------------------------------
component mon_rom
  Port (
    clk      : in  std_logic;
    rst      : in  std_logic;
    cs       : in  std_logic;
    rw       : in  std_logic;
    addr     : in  std_logic_vector (12 downto 0);
    data_in  : in  std_logic_vector (7 downto 0);
    data_out : out std_logic_vector (7 downto 0)
    );
end component;

component cart_rom
  Port (
    clk      : in  std_logic;
    rst      : in  std_logic;
    cs       : in  std_logic;
    rw       : in  std_logic;
    addr     : in  std_logic_vector (12 downto 0);
    data_in  : in  std_logic_vector (7 downto 0);
    data_out : out std_logic_vector (7 downto 0)
    );
end component;

component ram_2k
 Port (
    clk      : in  std_logic;
    rst      : in  std_logic;
    cs       : in  std_logic;
    rw       : in  std_logic;
    addr     : in  std_logic_vector (9 downto 0);
    data_in  : in  std_logic_vector (7 downto 0);
    data_out : out std_logic_vector (7 downto 0)
    );
end component;

component via6522 is
port (
    clock       : in  std_logic;
    clock_en    : in  std_logic; -- for counters and stuff
    reset       : in  std_logic;
    
    addr        : in  std_logic_vector(3 downto 0);
    wen         : in  std_logic;
    ren         : in  std_logic;
    data_in     : in  std_logic_vector(7 downto 0);
    data_out    : out std_logic_vector(7 downto 0);

    -- pio --
    port_a_o    : out std_logic_vector(7 downto 0);
    port_a_t    : out std_logic_vector(7 downto 0);
    port_a_i    : in  std_logic_vector(7 downto 0);
    
    port_b_o    : out std_logic_vector(7 downto 0);
    port_b_t    : out std_logic_vector(7 downto 0);
    port_b_i    : in  std_logic_vector(7 downto 0);

    -- handshake pins
    ca1_i       : in  std_logic;

    ca2_o       : out std_logic;
    ca2_i       : in  std_logic;
    ca2_t       : out std_logic;
    
    cb1_o       : out std_logic;
    cb1_i       : in  std_logic;
    cb1_t       : out std_logic;
    
    cb2_o       : out std_logic;
    cb2_i       : in  std_logic;
    cb2_t       : out std_logic;

    irq         : out std_logic 
      );
end component;


component glasses3d is
--The pos values are in range 0-179, with the amount of degrees as index. 180-360 are 'black'.
	generic (
		pos_green: integer :=0;
		pos_red: integer :=110;
		pos_blue: integer :=130;
		pos_index: integer :=90
   );
    Port ( 
			  clk : in  STD_LOGIC; --12MHz please
           reset : in  STD_LOGIC;
           pwm_in : in  STD_LOGIC;
           sync_out : out  STD_LOGIC;
           rgb_out : out  STD_LOGIC_VECTOR (2 downto 0);
           shutter_out : out  STD_LOGIC
);
end component;


component signed_integ is
   generic(
    data_width: integer := 8;
    intern_data_width: integer := 10;
	 output_data_width: integer := 10
    );
 port(
      clk_i              :  in  std_logic;
      rst_i              :  in  std_logic;
      ramp_i			 :  in  std_logic;
      zero_i			 :  in  std_logic;
      load_offset_i		 :  in  std_logic;
      data_i          	 :  in  std_logic_vector(data_width-1 downto 0);
      offset_i			 :  in  std_logic_vector(data_width-1 downto 0);
      data_o        	 :  out std_logic_vector(output_data_width-1 downto 0);
      data_o_stb		 :  out std_logic
  );
end component;

component YM2149 is
	Port (
		  -- data bus
		  I_DA                : in  std_logic_vector(7 downto 0);
		  O_DA                : out std_logic_vector(7 downto 0);
		  O_DA_OE_L           : out std_logic;
		  -- control
		  I_A9_L              : in  std_logic;
		  I_A8                : in  std_logic;
		  I_BDIR              : in  std_logic;
		  I_BC2               : in  std_logic;
		  I_BC1               : in  std_logic;
		  I_SEL_L             : in  std_logic;

		  O_AUDIO             : out std_logic_vector(7 downto 0);
		  -- port a
		  I_IOA               : in  std_logic_vector(7 downto 0);
		  O_IOA               : out std_logic_vector(7 downto 0);
		  O_IOA_OE_L          : out std_logic;
		  -- port b
		  I_IOB               : in  std_logic_vector(7 downto 0);
		  O_IOB               : out std_logic_vector(7 downto 0);
		  O_IOB_OE_L          : out std_logic;
		  --
		  ENA                 : in  std_logic; -- clock enable
		  RESET_L             : in  std_logic; -- E clock , 1.5MHz
		  CLK                 : in  std_logic  -- 6 Mhz
		);
end component;


end pkg_vectrex;
