library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.NUMERIC_STD.ALL;

use ieee.std_logic_unsigned.all;

entity signed_integ is
   generic(
    data_width: integer := 8;
    intern_data_width: integer := 19;
    output_data_width: integer := 10
    );
 
port(
      clk_i              :  in  std_logic;
      rst_i              :  in  std_logic;
      ramp_i			 :  in  std_logic;
      zero_i			 :  in  std_logic;
      data_i          	 :  in  std_logic_vector(data_width-1 downto 0);
      offset_i			 :  in  std_logic_vector(data_width-1 downto 0);
      data_o        	 :  out std_logic_vector(output_data_width-1 downto 0)
  );
end signed_integ;




----------------
architecture RTL of signed_integ is
	signal pos: STD_LOGIC_VECTOR(intern_data_width-1 downto 0);
begin

  process(clk_i,data_i) is
		variable newPos: STD_LOGIC_VECTOR(intern_data_width-1 downto 0);
  begin
    if rising_edge(clk_i) then
		if zero_i='1' then
			pos<=(others => '0');
		elsif ramp_i = '1' then
			newPos:=pos+sxt(data_i, newPos'length)-sxt(offset_i, newPos'length);
			--Saturate result, so we don't wrap around
			if newPos(intern_data_width-1 downto intern_data_width-2)="01" then
				--Positive overflow.
				newPos(intern_data_width-1 downto intern_data_width-2):="00";
				newPos(intern_data_width-3 downto 0):=(others => '1');
			elsif newPos(intern_data_width-1 downto intern_data_width-2)="10" then
				--Negative overflow.
				newPos(intern_data_width-1 downto intern_data_width-2):="11";
				newPos(intern_data_width-2 downto 0):=(others => '0');
			end if;
			pos<=newPos;
		end if;
	end if;
  end process;
  data_o <= pos(intern_data_width-2 downto intern_data_width-output_data_width-1);
end;
